@@ -1,19 +1,17 @@
 # -*- Perl -*-
 #
-# run unix commands for expected results given particular inputs
+# run simple unix commands for expected results given particular inputs
 
 package Test2::Tools::Command;
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 use 5.10.0;
 use strict;
 use warnings;
-use Cwd ();
 use File::chdir;    # $CWD
 use IPC::Open3 'open3';
 use Symbol 'gensym';
 use Test2::API 'context';
-use Test2::Tools::Compare;
 
 use base 'Exporter';
 our @EXPORT = qw(command);
@@ -25,6 +23,7 @@ sub command ($) {
     local $CWD = $_[0]->{chdir} if defined $_[0]->{chdir};
     local @ENV{ keys %{ $_[0]->{env} } } = values %{ $_[0]->{env} };
 
+    # what to run, and possibly also a string to include in the test name
     my @cmd = ( @command, exists $_[0]->{args} ? @{ $_[0]->{args} } : () );
 
     my ( $stdout, $stderr );
@@ -40,6 +39,7 @@ sub command ($) {
             print $in $_[0]->{stdin};
             close $in;
         }
+        # this may be bad if the utility produces too much output
         $stdout = do { local $/; readline $out };
         $stderr = do { local $/; readline $err };
         waitpid $pid, 0;
@@ -49,14 +49,16 @@ sub command ($) {
     my $orig_status = $?;
     # the exit status is broken out into a hashref for exact tests on
     # the various components of the 16-bit word (an alternative might be
-    # to mangle the value like the shell does)
+    # to mangle it into a number like the shell does)
     my $status = {
         code   => $? >> 8,
         signal => $? & 127,
         iscore => $? & 128 ? 1 : 0
     };
-    # the munge are for when the code or signal vary and you only want
-    # to know if the value was 0 or not
+    # the munge are for when the code or signal vary (for portability or
+    # for reasons out of your control) and you only want to know if the
+    # value was 0 or not. lots of CPAN Tester systems did not set the
+    # iscore flag following a CORE::dump by a test program...
     $status->{code}   = $status->{code}   ? 1 : 0 if $_[0]->{munge_status};
     $status->{signal} = $status->{signal} ? 1 : 0 if $_[0]->{munge_signal};
 
@@ -72,19 +74,60 @@ sub command ($) {
         $_[0]->{status} = { code => 0, signal => 0, iscore => 0 };
     }
 
-    my $ctx    = context();
-    my $name   = $_[0]->{name} // "@cmd";
-    my $result = is $status, $_[0]->{status}, "exit - $name";
-    # qr// or assume it's a string
-    if (defined $_[0]->{stdout} and ref $_[0]->{stdout} eq 'Regexp') {
-        $result ||= like $stdout, $_[0]->{stdout}, "stdout - $name";
+    my ( $ctx, $name, $result ) = ( context(), $_[0]->{name} // "@cmd", 1 );
+
+    if (    $_[0]->{status}{code} == $status->{code}
+        and $_[0]->{status}{signal} == $status->{signal}
+        and $_[0]->{status}{iscore} == $status->{iscore} ) {
+        $ctx->pass("exit - $name");
     } else {
-        $result ||= is $stdout, $_[0]->{stdout} // '', "stdout - $name";
+        $ctx->fail(
+            "exit - $name",
+            sprintf(
+                "code\t%d\tsignal\t%d\tiscore\t%d want",
+                $_[0]->{status}{code},
+                $_[0]->{status}{signal},
+                $_[0]->{status}{iscore}
+            ),
+            sprintf(
+                "code\t%d\tsignal\t%d\tiscore\t%d got",
+                $status->{code}, $status->{signal}, $status->{iscore}
+            )
+        );
+        $result = 0;
     }
-    if (defined $_[0]->{stderr} and ref $_[0]->{stderr} eq 'Regexp') {
-        $result ||= like $stderr, $_[0]->{stderr}, "stderr - $name";
+    # qr// or assume it's a string
+    if ( defined $_[0]->{stdout} and ref $_[0]->{stdout} eq 'Regexp' ) {
+        if ( $stdout =~ m/$_[0]->{stdout}/ ) {
+            $ctx->pass("stdout - $name");
+        } else {
+            $ctx->fail( "stdout - $name", "expected match on $_[0]->{stdout}" );
+            $result = 0;
+        }
     } else {
-        $result ||= is $stderr, $_[0]->{stderr} // '', "stderr - $name";
+        my $want = $_[0]->{stdout} // '';
+        if ( $stdout eq $want ) {
+            $ctx->pass("stdout - $name");
+        } else {
+            $ctx->fail( "stdout - $name", "expected equality with q{$want}" );
+            $result = 0;
+        }
+    }
+    if ( defined $_[0]->{stderr} and ref $_[0]->{stderr} eq 'Regexp' ) {
+        if ( $stderr =~ m/$_[0]->{stderr}/ ) {
+            $ctx->pass("stderr - $name");
+        } else {
+            $ctx->fail( "stderr - $name", "expected match on $_[0]->{stderr}" );
+            $result = 0;
+        }
+    } else {
+        my $want = $_[0]->{stderr} // '';
+        if ( $stderr eq $want ) {
+            $ctx->pass("stderr - $name");
+        } else {
+            $ctx->fail( "stderr - $name", "expected equality with q{$want}" );
+            $result = 0;
+        }
     }
     $ctx->release;
     return $result, $orig_status, \$stdout, \$stderr;
@@ -99,17 +142,28 @@ Test2::Tools::Command - test simple unix commands
 
 =head1 SYNOPSIS
 
-  use Test2::V0;
   use Test2::Tools::Command;
 
+  # test some typical unix tools; implicit checks are that status
+  # is 0, and that stdout and stderr are the empty string, unless
+  # otherwise specified
+  command { args => [ 'true'        ] };
+  command { args => [ 'false'       ], status => 1 };
+  command { args => [ 'echo', 'foo' ], stdout => "foo\n" };
+
   # subsequent args are prefixed with this
   local @Test2::Tools::Command::command = ( 'perl', '-E' );
 
-  command { args   => [ q{say "out"; exit 42} ],
-            chdir  => '/some/dir',
-            stdin  => "printed to program\n",
-            stdout => qr/out/,
-            status => 42 };
+  # return values and a variety of the options available
+  my ($result, $exit_status, $stdout_ref, $stderr_ref) =
+   command { args    => [ q{say "out";warn "err";kill TERM => $$} ],
+             chdir   => '/some/dir',
+             env     => { API_KEY => 42 },
+             stdin   => "printed to program\n",
+             stdout  => qr/out/,
+             stderr  => qr/err/,
+             status  => { code => 0, signal => 15, iscore => 0 },
+             timeout => 7 };
 
 =head1 DESCRIPTION
 
